--author: igor29381
--v2.1
--date 14.06.17

GarageManager = {};
GarageManager.curModDir = g_currentModDirectory;

function GarageManager:loadMap(name)
	self.unknownVehicleImage = createImageOverlay(Utils.getFilename("textures/hud_pda_spot_helper.dds", GarageManager.curModDir));
	self.garages = {};
	self.garageSaves = {};
	self.garageStartPos = 1;
	self.loaded = false;
	g_currentMission.environment:addDayChangeListener(self);
end;

function GarageManager:loadData()
	if g_server then
		local xmlPath = "PlaceableGarageSaves.xml";
		if g_currentMission.missionInfo.savegameDirectory then
			xmlPath = g_currentMission.missionInfo.savegameDirectory .. "/PlaceableGarageSaves.xml";
		end;
		if fileExists(xmlPath) then
			local xmlFile = loadXMLFile("PlaceableGarageSaves", xmlPath);
			local i = 0;
			while true do
				local key = string.format("vehicles.vehicle(%d)", i);
				if not hasXMLProperty(xmlFile, key) then break; end;
				local vehicleXML = Utils.convertFromNetworkFilename(getXMLString(xmlFile, key.."#filename"));
				if vehicleXML then
					g_currentMission:loadVehicle(vehicleXML, 0, 0, 0, 0, true, 0, Vehicle.PROPERTY_STATE_OWNED, nil, {xmlFile=xmlFile, key=key, resetVehicles=false});
				end;
				i = i + 1;
			end;
			delete(xmlFile);
		end;
		self.loaded = true;
	end;
end;

function GarageManager:dayChanged()
	if g_server then
		local totalCosts = 0;
		for _,garage in pairs(self.garages) do
			if garage.storeNums and #garage.storeNums > 0 then
				for ii=1, #garage.storeNums do
					local num = garage.storeNums[ii]
					local storeItem = StoreItemsUtil.storeItems[num];
					if storeItem then
						totalCosts = totalCosts + storeItem.dailyUpkeep;
					end;
				end;
			end;
		end;
		g_currentMission:addSharedMoney(-totalCosts, "other");
		g_currentMission:addMoneyChange(-totalCosts, FSBaseMission.MONEY_TYPE_SINGLE, true);
	end;
end;

function GarageManager:deleteMap()
	self.garages = {};
	self.garageOverlays = nil;
	self.garageTexts = nil;
	if g_currentMission.environment then
		g_currentMission.environment:removeDayChangeListener(self);
	end;
end;

function GarageManager:mouseEvent(posX, posY, isDown, isUp, button)
	if self.garageOverlays and #self.garageOverlays > 7 then
		if button == Input.MOUSE_BUTTON_WHEEL_UP then
			self.garageStartPos = math.max(self.garageStartPos - 0.5, 1);
		elseif button == Input.MOUSE_BUTTON_WHEEL_DOWN then
			self.garageStartPos = math.min(self.garageStartPos + 0.5, #self.garageOverlays - 6);
		end;
	end;
end;

function GarageManager:keyEvent(unicode, sym, modifier, isDown)
end;

function GarageManager:update(dt)
end;

function GarageManager:draw()
	if self.garageOverlays then
		for i=1, math.min(7, #self.garageOverlays) do
			local position = math.floor(self.garageStartPos-1)+i;
			if self.garageOverlays[position] and self.garageTexts[position] then
				renderOverlay(self.garageOverlays[position], 0.46, 0.2 + 0.1*i, 0.08, 0.08);
				setTextColor(1, 1, 1, 1);
				setTextAlignment(RenderText.ALIGN_CENTER);
				renderText(0.5, 0.2 + 0.1*i, 0.02, self.garageTexts[position]);
				setTextAlignment(RenderText.ALIGN_LEFT);
			end;
		end;
	end;
end;

function GarageManager:save()
	local garagesString = "";
	for _,garage in pairs(self.garages) do
		if garage.vehiclesString ~= "" then
			if garagesString == "" then
				garagesString = garage.vehiclesString;
			else
				garagesString = string.format("%s\n%s", garagesString, garage.vehiclesString);
			end;
		end;
	end;
	if garagesString ~= "" then
		local file = io.open(string.format("%s/PlaceableGarageSaves.xml", g_currentMission.missionInfo.savegameDirectory), "w");
		file:write('<?xml version="1.0" encoding="utf-8" standalone="no" ?>\n<vehicles>\n', garagesString, '\n</vehicles>');
		file:close();
	end;
end;

addModEventListener(GarageManager);
