--author: igor29381
--v2.1
--date 23.06.17

GaragePlaceable = {};
GaragePlaceable_mt = Class(GaragePlaceable, Placeable);
InitObjectClass(GaragePlaceable, "GaragePlaceable");

GaragePlaceable.STATE_CLOSED = 1;
GaragePlaceable.STATE_OPENING = 2;
GaragePlaceable.STATE_OPENED = 3;
GaragePlaceable.STATE_CLOSING = 4;

function GaragePlaceable:new(isServer, isClient, customMt)
	local mt = customMt;
	if mt == nil then
		mt = GaragePlaceable_mt;
	end;
	local self = Placeable:new(isServer, isClient, mt);
	registerObjectClassName(self, "GaragePlaceable");
	return self;
end;

function GaragePlaceable:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)
	if not GaragePlaceable:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
		return false;
	end;
	if x ~= 0 and z ~= 0 then
		self.triggerId = getChild(self.nodeId, "playerTrigger");
		addTrigger(self.triggerId, "triggerCallback", self);
		self.vehicleTriggerId = getChild(self.nodeId, "vehicleTrigger");
		addTrigger(self.vehicleTriggerId, "vehicleTriggerCallback", self);
		self.gates = {};
		local gatesNode = getChild(self.nodeId, "gates");
		if gatesNode > 0 then
			local gatesString = getUserAttribute(gatesNode, "gates");
			if gatesString then
				local gates = Utils.splitString(" ", gatesString);
				for i=1, #gates do
					local gate = {};
					gate.node = Utils.indexToObject(gatesNode, gates[i]);
					gate.startX, gate.startY, gate.startZ = getRotation(gate.node);
					gate.endX = gate.startX + math.rad(Utils.getNoNil(getUserAttribute(gate.node, "rotX"), "0"));
					gate.endY = gate.startY + math.rad(Utils.getNoNil(getUserAttribute(gate.node, "rotY"), "0"));
					gate.endZ = gate.startZ + math.rad(Utils.getNoNil(getUserAttribute(gate.node, "rotZ"), "0"));
					table.insert(self.gates, gate);
				end;
			else
				for g=1, getNumOfChildren(gatesNode) do
					local gate = {};
					gate.node = getChildAt(gatesNode, g-1);
					gate.startX, gate.startY, gate.startZ = getRotation(gate.node);
					gate.endX = gate.startX + math.rad(Utils.getNoNil(getUserAttribute(gate.node, "rotX"), "0"));
					gate.endY = gate.startY + math.rad(Utils.getNoNil(getUserAttribute(gate.node, "rotY"), "0"));
					gate.endZ = gate.startZ + math.rad(Utils.getNoNil(getUserAttribute(gate.node, "rotZ"), "0"));
					table.insert(self.gates, gate);
				end;
			end;
		end;
		local rollGatesNode = getChild(self.nodeId, "rollGates");
		if rollGatesNode > 0 then
			self.rollGates = {};
			for g=1, getNumOfChildren(rollGatesNode) do
				table.insert(self.rollGates, getChildAt(rollGatesNode, g-1));
			end;
		end;
		local transGatesNode = getChild(self.nodeId, "transGates");
		if transGatesNode > 0 then
			self.transGates = {};
			for g=1, getNumOfChildren(transGatesNode) do
				local gate = {};
				gate.node = getChildAt(transGatesNode, g-1);
				gate.startX, gate.startY, gate.startZ = getTranslation(gate.node);
				gate.endX = gate.startX + Utils.getNoNil(getUserAttribute(gate.node, "transX"), "0");
				gate.endY = gate.startY + Utils.getNoNil(getUserAttribute(gate.node, "transY"), "0");
				gate.endZ = gate.startZ + Utils.getNoNil(getUserAttribute(gate.node, "transZ"), "0");
				table.insert(self.transGates, gate);
			end;
		end;
		self.startPoint = getChild(self.nodeId, "startPoint");
		local endPoint = getChild(self.startPoint, "endPoint");
		if endPoint > 0 then
			local ex,_,ez = getTranslation(endPoint);
			self.dimensions = {ex, ez};
		end;
		local visitPoint = getChild(self.nodeId, "visitPoint");
		if visitPoint > 0 then
			x, _, z = getWorldTranslation(visitPoint);
		end;
		local width, height = getNormalizedScreenValues(10, 10);
		local uvs = getNormalizedUVs({135, 940, 46, 46});
		self.mapHotspot = g_currentMission.ingameMap:createMapHotspot("", "", g_baseUIFilename, uvs, nil, x, z, width, height, false, false, true, nil, true);
		self.sound = getChild(self.nodeId, "sound");
		if self.sound > 0 then setVisibility(self.sound, false); end;
		self.light = getChild(self.nodeId, "light");
		if self.light > 0 then setVisibility(self.light, false); end;
		self.vehiclesString = "";
		self.state = GaragePlaceable.STATE_CLOSED;
		self.gatePos = 0;
		self.speed = Utils.getNoNil(getUserAttribute(self.nodeId, "speed"), 0.0003);
		self.storeNums = {};
		self.storeTexts = {};
		self.storeImages = {};
		self.player = false;
		self.isEnabled = true;
		GarageManager.garages[self.id] = self;
	end;
	return true;
end;

function GaragePlaceable:delete()
	if self.mapHotspot then
		g_currentMission.ingameMap:deleteMapHotspot(self.mapHotspot);
	end;
	if self.triggerId then
		removeTrigger(self.triggerId);
	end;
	if self.vehicleTriggerId then
		removeTrigger(self.vehicleTriggerId);
	end;
	unregisterObjectClassName(self);
	GaragePlaceable:superClass().delete(self);
end;

function GaragePlaceable:deleteFinal()
	GaragePlaceable:superClass().deleteFinal(self);
end;

function GaragePlaceable:readStream(streamId, connection)
	GaragePlaceable:superClass().readStream(self, streamId, connection);
	self.state = streamReadInt8(streamId);
	local count = streamReadInt8(streamId);
	if count > 0 then
		for i=1, count do
			local storeNum = streamReadInt32(streamId);
			self:insertStoreItems(storeNum);
		end;
	else
		GarageManager.garageOverlays = nil;
		GarageManager.garageTexts = nil;
	end;
end;

function GaragePlaceable:writeStream(streamId, connection)
	GaragePlaceable:superClass().writeStream(self, streamId, connection);
	local state = GaragePlaceable.STATE_CLOSED;
	if self.state == GaragePlaceable.STATE_OPENED or self.state == GaragePlaceable.STATE_OPENING then
		state = GaragePlaceable.STATE_OPENING;
	end;
	streamWriteInt8(streamId, state);
	local count = #self.storeNums;
	streamWriteInt8(streamId, count);
	if count > 0 then
		for i=1, count do
			streamWriteInt32(streamId, self.storeNums[i]);
		end;
	end;
end;

function GaragePlaceable:update(dt)
	if self.player then
		setTextColor(1, 1, 1, 1);
		setTextAlignment(RenderText.ALIGN_CENTER);
		if self.state == GaragePlaceable.STATE_OPENED or self.state == GaragePlaceable.STATE_OPENING then
			renderText(0.5, 0.28, 0.02, g_i18n:getText("shutGates"));
		elseif self.state == GaragePlaceable.STATE_CLOSED or self.state == GaragePlaceable.STATE_CLOSING then
			renderText(0.5, 0.28, 0.02, g_i18n:getText("openGates"));
		end;
		if #self.storeTexts > 0 then
			GarageManager.garageOverlays = self.storeImages;
			GarageManager.garageTexts = self.storeTexts;
		else
			renderText(0.5, 0.3, 0.02, g_i18n:getText("empty"));
		end;
		setTextAlignment(RenderText.ALIGN_LEFT);
		if InputBinding.hasEvent(InputBinding.GARAGE_DOORS) then
			if self.isServer then
				self:onChangeState();
			else
				g_client:getServerConnection():sendEvent(GaragePlaceableEvent:new(true, 0, 0, self));
			end;
		end;
	end;
	if self.state == GaragePlaceable.STATE_OPENING then
		self.gatePos = math.min(self.gatePos + self.speed * dt, 1);
		if self.gatePos == 1 then
			self.state = GaragePlaceable.STATE_OPENED;
			if self.sound > 0 then setVisibility(self.sound, false); end;
			if self.light > 0 then setVisibility(self.light, true); end;
		end;
	end;
	if self.state == GaragePlaceable.STATE_CLOSING then
		self.gatePos = math.max(self.gatePos - self.speed * dt, 0);
		if self.gatePos == 0 then
			self.state = GaragePlaceable.STATE_CLOSED;
			if self.sound > 0 then setVisibility(self.sound, false); end;
			if self.isServer then self:onEndClosing(true); end;
		end;
	end;
	if self.gatePos > 0 and self.gatePos < 1 then
		for _, gate in pairs(self.gates) do
			local x = (1 - self.gatePos) * gate.startX + self.gatePos * gate.endX;
			local y = (1 - self.gatePos) * gate.startY + self.gatePos * gate.endY;
			local z = (1 - self.gatePos) * gate.startZ + self.gatePos * gate.endZ;
			setRotation(gate.node, x, y, z);
		end;
		if self.rollGates then
			self:updateRollGates();
		end;
		if self.transGates then
			for _, gate in pairs(self.transGates) do
				local x = (1 - self.gatePos) * gate.startX + self.gatePos * gate.endX;
				local y = (1 - self.gatePos) * gate.startY + self.gatePos * gate.endY;
				local z = (1 - self.gatePos) * gate.startZ + self.gatePos * gate.endZ;
				setTranslation(gate.node, x, y, z);
			end;
		end;
		if self.sound > 0 then setVisibility(self.sound, true); end;
	end;
end;

function GaragePlaceable:updateRollGates()
	for _, gateNode in pairs(self.rollGates) do
		local pos = 1-self.gatePos;
		setScale(gateNode, 1, pos, 1);
		setShaderParameter(gateNode, "mirrorScaleAndOffsetUV", 1, pos*0.5, 0, 0, false);
	end;
end;

function GaragePlaceable:onChangeState()
	if self.state == GaragePlaceable.STATE_CLOSED then
		self.state = GaragePlaceable.STATE_OPENING;
		self:onStartOpening();
	elseif self.state == GaragePlaceable.STATE_OPENED then
		if self.light > 0 then setVisibility(self.light, false); end;
		self.state = GaragePlaceable.STATE_CLOSING;
		g_server:broadcastEvent(GaragePlaceableEvent:new(false, GaragePlaceable.STATE_CLOSING, 0, self));
	end;
end;

function GaragePlaceable:onStartOpening()
	if self.vehiclesString ~= "" then
		local folderName = g_currentMission.missionInfo.savegameDirectory;
		if not folderName then
			folderName = string.format("%s/savegame%d", getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex);
			createFolder(folderName);
		end;
		local fileName = string.format("%s/VehicleInGarage.xml", folderName);
		local file = io.open(fileName, "w");
		file:write('<?xml version="1.0" encoding="utf-8" standalone="no" ?>\n<vehicles>\n', self.vehiclesString, '\n</vehicles>');
		file:close();
		local xmlFile = loadXMLFile("tempXML", fileName);
		local i = 0;
		while true do
			local key = string.format("vehicles.vehicle(%d)", i);
			if not hasXMLProperty(xmlFile, key) then break; end;
			local vehicleXML = Utils.convertFromNetworkFilename(getXMLString(xmlFile, key.."#filename"));
			if vehicleXML then
				local x,_,z = getWorldTranslation(self.nodeId);
				g_currentMission:loadVehicle(vehicleXML, x, 0.2, z, 0, true, 0, 0, nil, {xmlFile=xmlFile, key=key, resetVehicles=false});
			end;
			i=i+1;
		end;
		delete(xmlFile);
		self.vehiclesString = "";
		GarageManager.garageOverlays = nil;
		GarageManager.garageTexts = nil;
		self.storeNums = {};
		self.storeTexts = {};
		self.storeImages = {};
	end;
	g_server:broadcastEvent(GaragePlaceableEvent:new(false, GaragePlaceable.STATE_OPENING, 0, self));
end;

function GaragePlaceable:onEndClosing(sendEvent)
	local vehicles = {};
	for i=1, #g_currentMission.vehicles do
		local vehicle = g_currentMission.vehicles[i];
		if self.dimensions and not (vehicle.leasingInfos and vehicle.leasingInfos.leased) then
			local vx1, _, vz1 = worldToLocal(self.startPoint, getWorldTranslation(vehicle.rootNode));
			if vx1 > 0 and vz1 > 0 and vx1 < self.dimensions[1] and vz1 < self.dimensions[2] then
				table.insert(vehicles, vehicle);
			end;
		end;
	end;
	if #vehicles > 0 then
		local vehiclesString = "";
		for i=1, #vehicles do
			local vehicle = vehicles[i];
			local attributes, nodes = vehicle:getSaveAttributesAndNodes("       ");
			attributes = string.format('<vehicle filename="%s" %s>%s\n</vehicle>', Utils.encodeToHTML(Utils.convertToNetworkFilename(vehicle.configFileName)), attributes, nodes);
			if vehiclesString == "" then
				vehiclesString = attributes;
			else
				vehiclesString = string.format("%s\n%s", vehiclesString, attributes);
			end;
			self.vehiclesString = vehiclesString;
			local storeNum = StoreItemsUtil.storeItemsByXMLFilename[string.lower(vehicle.configFileName)].id;
			self:insertStoreItems(storeNum);
			g_currentMission:removeVehicle(vehicle);
			if sendEvent then
				g_server:broadcastEvent(GaragePlaceableEvent:new(false, 0, storeNum, self));
			end;
		end;
	end;
end;

function GaragePlaceable:insertStoreItems(storeNum)
	local storeItem = StoreItemsUtil.storeItems[storeNum];
	local storeText = "Unknown vehicle";
	local storeImage = GarageManager.unknownVehicleImage;
	if storeItem then
		storeText = string.format("%s %s", storeItem.brand, storeItem.name);
		if storeItem.imageActive ~= "" then
			storeImageActive = createImageOverlay(Utils.getFilename(storeItem.imageActive));
			if storeImageActive and storeImageActive ~= 0 then
				storeImage = storeImageActive;
			end;
		end;
	end;
	table.insert(self.storeNums, storeNum);
	table.insert(self.storeTexts, storeText);
	table.insert(self.storeImages, storeImage);
end;

function GaragePlaceable:triggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
	if onEnter then
		if g_currentMission.player and g_currentMission.player.rootNode == otherShapeId then
			self.player = true;
		end;
	elseif onLeave then
		self.player = false;
		GarageManager.garageOverlays = nil;
		GarageManager.garageTexts = nil;
	end;
end;

function GaragePlaceable:vehicleTriggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
	if onEnter and self.isServer and self.state == GaragePlaceable.STATE_CLOSED and self.vehiclesString == "" then
		local vehicle = g_currentMission.nodeToVehicle[otherShapeId];
		if vehicle then
			self:onChangeState();
		end;
	end;
end;

function GaragePlaceable:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not GaragePlaceable:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false;
	end;
	self.state = Utils.getNoNil(getXMLInt(xmlFile, key.."#state"), 1);
	if self.state == GaragePlaceable.STATE_OPENED then
		self.gatePos = 1;
		for _, gate in pairs(self.gates) do
			setRotation(gate.node, gate.endX, gate.endY, gate.endZ);
		end;
		if self.rollGates then
			self:updateRollGates();
		end;
		if self.transGates then
			for _, gate in pairs(self.transGates) do
				setTranslation(gate.node, gate.endX, gate.endY, gate.endZ);
			end;
		end;
		if self.light > 0 then setVisibility(self.light, true); end;
	end;
	if not GarageManager.loaded then
		GarageManager:loadData();
	end;
	if self.state == GaragePlaceable.STATE_CLOSED then
		self:onEndClosing(false);
	end;
	return true;
end;

function GaragePlaceable:getSaveAttributesAndNodes(nodeIdent)
	local attributes, nodes = GaragePlaceable:superClass().getSaveAttributesAndNodes(self, nodeIdent);
	if table.maxn(GarageManager.garages) == self.id then
		GarageManager:save();
	end;
	attributes = attributes..' state="'..self.state..'"';
	return attributes, nodes;
end;

registerPlaceableType("GaragePlaceable", GaragePlaceable);
-------------------------------------------------------------------------------------------------
GaragePlaceableEvent = {};
GaragePlaceableEvent_mt = Class(GaragePlaceableEvent, Event);

InitEventClass(GaragePlaceableEvent, "GaragePlaceableEvent");

function GaragePlaceableEvent:emptyNew()
	local self = Event:new(GaragePlaceableEvent_mt);
    return self;
end;

function GaragePlaceableEvent:new(changeState, state, storeNum, object)
	local self = GaragePlaceableEvent:emptyNew();
	self.changeState = changeState;
	self.state = state;
	self.storeNum = storeNum;
	self.object = object;
	return self;
end;

function GaragePlaceableEvent:readStream(streamId, connection)
	local changeState = streamReadBool(streamId);
	local state = streamReadInt8(streamId);
	local storeNum = streamReadInt32(streamId);
	local id = streamReadInt32(streamId);
    local object = networkGetObject(id);
	if object ~= nil then
		if g_server then
			if changeState then
				object:onChangeState();
			end;
		else
			if storeNum > 0 then
				object:insertStoreItems(storeNum);
			else
				object.storeNums = {};
				object.storeTexts = {};
				object.storeImages = {};
				GarageManager.garageOverlays = nil;
				GarageManager.garageTexts = nil;
			end;
		end;
		if state > 0 then
			object.state = state;
			if state == GaragePlaceable.STATE_CLOSING then
				if object.light > 0 then setVisibility(object.light, false); end;
			end;
		end;
	end;
end;

function GaragePlaceableEvent:writeStream(streamId, connection)
	streamWriteBool(streamId, self.changeState);
	streamWriteInt8(streamId, self.state);
	streamWriteInt32(streamId, self.storeNum);
	streamWriteInt32(streamId, networkGetObjectId(self.object));
end;
